#include <libxml/tree.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include "partition.h"

static void add_attr_str(xmlNodePtr node, const char *name, const char *value)
{
	xmlNewProp(node, BAD_CAST name, BAD_CAST value);
}

static void add_attr_int(xmlNodePtr node, const char *name, int val)
{
	char str[32];

	if (val == 0)
		return;
	sprintf(str, "%d", val);
	xmlNewProp(node, BAD_CAST name, BAD_CAST str);
}

int pit2xml(const char *pitfile, const char *xmlfile)
{
	int fd, r, i, ret = -1;
	partition_header header;
	xmlDocPtr doc;
	xmlNodePtr root_node, node;

	fd = open(pitfile, O_RDONLY);
	if (fd < 0)
	{
		fprintf(stderr, "failed to open %s\n", pitfile);
		return 1;
	}

	doc = xmlNewDoc(BAD_CAST "1.0");
	root_node = xmlNewNode(NULL, BAD_CAST "root");
	xmlDocSetRootElement(doc, root_node);

	r = read(fd, &header, sizeof header);
	if (r != sizeof header)
		goto fail;

	if (header.dwMagic != PARTITION_MAGIC)
	{
		fprintf(stderr, "%s is not a PIT file\n", pitfile);
		goto fail;
	}

	for (i=0; i<header.nCount; i++)
	{
		partition_info part;

		r = read(fd, &part, sizeof part);
		if (r != sizeof part)
			goto fail;

		node = xmlNewChild(root_node, NULL, BAD_CAST "partition", NULL);

		add_attr_str(node, "Name", part.szName);
		add_attr_str(node, "FileName", part.szFileName);
		add_attr_str(node, "DeltaName", part.szDeltaName);
		add_attr_int(node, "BinType", part.nBinType);
		add_attr_int(node, "DevType", part.nDevType);
		add_attr_int(node, "ID", part.nID);
		add_attr_int(node, "Attribute", part.nAttribute);
		add_attr_int(node, "UpdateAttr", part.nUpdateAttr);
		add_attr_int(node, "BlockSize", part.dwBlkSize);
		add_attr_int(node, "BlockLength", part.dwBlkLen);
		add_attr_int(node, "Offset", part.dwOffset);
		add_attr_int(node, "FileSize", part.dwFileSize);
	}

	xmlSaveFormatFileEnc(xmlfile, doc, "UTF-8", 1);

	xmlFreeDoc(doc);

	ret = 0;

fail:
	close(fd);

	return ret;
}

int main(int argc, char **argv)
{

	if (argc != 3)
	{
		fprintf(stderr, "%s [pitfilename] [xmlfile]\n", argv[0]);
		return 1;
	}

	return pit2xml(argv[1], argv[2]);
}
